@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -35,21 +35,21 @@ void htaccess(const struct userinfostruct *uinfo)
 	FILE *fp_auth;
 	size_t i,nread;
 
-	if(!UserAuthentication)
+	if (!UserAuthentication)
 		return;
 
 	if (snprintf(htname,sizeof(htname),"%s/%s/.htaccess",outdirname,uinfo->filename)>=sizeof(htname)) {
-		debuga(_("Path too long: "));
+		debuga(__FILE__,__LINE__,_("Path too long: "));
 		debuga_more("%s/%s/.htaccess\n",outdirname,uinfo->filename);
 		exit(EXIT_FAILURE);
 	}
-	if((fp_auth=fopen(htname,"w"))==NULL) {
-		debugapos("auth",_("Cannot open file \"%s\": %s\n"),htname,strerror(errno));
+	if ((fp_auth=fopen(htname,"w"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),htname,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
 	if ((fp_in=fopen(AuthUserTemplateFile,"r"))==NULL) {
-		debugapos("auth",_("Cannot open file \"%s\": %s\n"),AuthUserTemplateFile,strerror(errno));
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),AuthUserTemplateFile,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -62,8 +62,14 @@ void htaccess(const struct userinfostruct *uinfo)
 				fputc(line[i],fp_auth);
 			}
 	}
-	fclose(fp_auth);
-	fclose(fp_in);
+	if (fclose(fp_auth)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),htname,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
+	if (fclose(fp_in)==EOF) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),AuthUserTemplateFile,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 
 	return;
 }
