@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -27,9 +27,9 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
-void convlog(const char *arq, char *df, int dfrom, int duntil)
+void convlog(const char *arq, char df, const struct ReadLogDataStruct *ReadFilter)
 {
-	FILE *fp_in;
+	FileObject *fp_in;
 	char *buf;
 	char data[30];
 	char dia[11];
@@ -39,46 +39,54 @@ void convlog(const char *arq, char *df, int dfrom, int duntil)
 	struct getwordstruct gwarea;
 	longline line;
 
-	if(arq[0] == '\0')
+	if (arq[0] == '\0')
 		arq="/var/log/squid/access.log";
 
-	if (arq[0]=='-' && arq[1]=='\0') {
-		fp_in=stdin;
-	} else if((fp_in=MY_FOPEN(arq,"r"))==NULL) {
-		debugapos("convlog",_("Cannot open file \"%s\": %s\n"),arq,strerror(errno));
+	if ((fp_in=FileObject_Open(arq))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),arq,FileObject_GetLastOpenError());
 		exit(EXIT_FAILURE);
 	}
 
 	if ((line=longline_create())==NULL) {
-		debuga(_("Not enough memory to read file \"%s\"\n"),arq);
+		debuga(__FILE__,__LINE__,_("Not enough memory to read file \"%s\"\n"),arq);
 		exit(EXIT_FAILURE);
 	}
 
 	while((buf=longline_read(fp_in,line))!=NULL) {
 		getword_start(&gwarea,buf);
 		if (getword(data,sizeof(data),&gwarea,' ')<0) {
-			debuga(_("Invalid record in file \"%s\"\n"),arq);
+			debuga(__FILE__,__LINE__,_("Invalid record in file \"%s\"\n"),arq);
 			exit(EXIT_FAILURE);
 		}
 		tt=atoi(data);
 		t=localtime(&tt);
 
-		if(dfrom) {
+		if (ReadFilter->DateRange[0])
+		{
 			idata=(t->tm_year+1900)*10000+(t->tm_mon+1)*100+t->tm_mday;
-			if(idata < dfrom || idata > duntil)
+			if (idata<ReadFilter->StartDate || idata>ReadFilter->EndDate)
+				continue;
+		}
+		if (ReadFilter->StartTime>=0 || ReadFilter->EndTime>=0)
+		{
+			int hmr=t->tm_hour*100+t->tm_min;
+			if (hmr<ReadFilter->StartTime || hmr>=ReadFilter->EndTime)
 				continue;
 		}
 
-		if(df[0]=='e')
+		if (df=='e')
 			strftime(dia, sizeof(dia), "%d/%m/%Y", t);
-		else
+		else if (df=='u')
 			strftime(dia, sizeof(dia), "%m/%d/%Y", t);
+		else //if (df=='w')
+			strftime(dia, sizeof(dia), "%Y.%U", t);
 
 		printf("%s %02d:%02d:%02d %s\n",dia,t->tm_hour,t->tm_min,t->tm_sec,gwarea.current);
 	}
 
 	longline_destroy(&line);
-	if (fp_in!=stdin && fclose(fp_in)==EOF) {
-		debuga(_("Failed to close file \"%s\": %s\n"),arq,strerror(errno));
+	if (FileObject_Close(fp_in)) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),arq,FileObject_GetLastCloseError());
+		exit(EXIT_FAILURE);
 	}
 }
