@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -58,23 +58,23 @@ void dansguardian_report(void)
 
 	ouser[0]='\0';
 
-	snprintf(dansguardian_in,sizeof(dansguardian_in),"%s/dansguardian.int_log",tmp);
-	if(!dansguardian_count) {
+	format_path(__FILE__, __LINE__, dansguardian_in, sizeof(dansguardian_in), "%s/dansguardian.int_log", tmp);
+	if (!dansguardian_count) {
 		if (!KeepTempLog && unlink(dansguardian_in))
-			debuga(_("Cannot delete \"%s\": %s\n"),dansguardian_in,strerror(errno));
-		if (debugz) debugaz(_("Dansguardian report not generated because it is empty\n"));
+			debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),dansguardian_in,strerror(errno));
+		if (debugz>=LogLevel_Process) debugaz(__FILE__,__LINE__,_("Dansguardian report not generated because it is empty\n"));
 		return;
 	}
 
-	sprintf(report,"%s/dansguardian.html",outdirname);
+	format_path(__FILE__, __LINE__, report, sizeof(report), "%s/dansguardian.html", outdirname);
 
-	if((fp_in=MY_FOPEN(dansguardian_in,"r"))==NULL) {
-		debugapos("dansguardian_report",_("Cannot open file \"%s\": %s\n"),dansguardian_in,strerror(errno));
+	if ((fp_in=MY_FOPEN(dansguardian_in,"r"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),dansguardian_in,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	if((fp_ou=MY_FOPEN(report,"w"))==NULL) {
-		debugapos("dansguardian_report",_("Cannot open file \"%s\": %s\n"),report,strerror(errno));
+	if ((fp_ou=MY_FOPEN(report,"w"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),report,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -92,23 +92,23 @@ void dansguardian_report(void)
 		getword_start(&gwarea,buf);
 		if (getword(user,sizeof(user),&gwarea,'\t')<0 || getword(date2,sizeof(date2),&gwarea,'\t')<0 ||
 		    getword(hour,sizeof(hour),&gwarea,'\t')<0 || getword(ip,sizeof(ip),&gwarea,'\t')<0) {
-			debuga(_("Invalid record in file \"%s\"\n"),dansguardian_in);
+			debuga(__FILE__,__LINE__,_("Invalid record in file \"%s\"\n"),dansguardian_in);
 			exit(EXIT_FAILURE);
 		}
 		if (getword_ptr(buf,&url,&gwarea,'\t')<0) {
-			debuga(_("Invalid url in file \"%s\"\n"),dansguardian_in);
+			debuga(__FILE__,__LINE__,_("Invalid url in file \"%s\"\n"),dansguardian_in);
 			exit(EXIT_FAILURE);
 		}
 		if (getword(rule,sizeof(rule),&gwarea,'\n')<0) {
-			debuga(_("Invalid rule in file \"%s\"\n"),dansguardian_in);
+			debuga(__FILE__,__LINE__,_("Invalid rule in file \"%s\"\n"),dansguardian_in);
 			exit(EXIT_FAILURE);
 		}
 
-		if(UserIp)
+		if (UserIp)
 			strcpy(user,ip);
 
-		bzero(date, 15);
-		if(strncmp(df,"u",1) != 0) {
+		memset(date,0,sizeof(date));
+		if (df!='u') {
 			strncpy(date,date2+6,2);
 			strcat(date,"/");
 			strncat(date,date2+4,2);
@@ -122,36 +122,36 @@ void dansguardian_report(void)
 			strncat(date,date2,4);
 		}
 
-		if(Ip2Name)
+		if (Ip2Name)
 			ip2name(ip,sizeof(ip));
 
-		if(!z) {
+		if (!z) {
 			strcpy(ouser,user);
 			strcpy(oip,ip);
 			z++;
 		} else {
-			if(strcmp(ouser,user) == 0)
+			if (strcmp(ouser,user) == 0)
 				user[0]='\0';
-			if(user[0] != '\0')
+			if (user[0] != '\0')
 				strcpy(ouser,user);
-			if(strcmp(oip,ip) == 0)
+			if (strcmp(oip,ip) == 0)
 				ip[0]='\0';
-			if(ip[0] != '\0')
+			if (ip[0] != '\0')
 				strcpy(oip,ip);
 		}
 
 		user_find(name, sizeof(name), user);
 
-		if(DansGuardianReportLimit) {
-			if(strcmp(ouser2,name) == 0) {
+		if (DansGuardianReportLimit) {
+			if (strcmp(ouser2,name) == 0) {
 				count++;
 			} else {
-				if(count>DansGuardianReportLimit && DansGuardianReportLimit>0)
+				if (count>DansGuardianReportLimit && DansGuardianReportLimit>0)
 					show_ignored_dansguardian(fp_ou,count-DansGuardianReportLimit);
 				count=1;
 				strcpy(ouser2,name);
 			}
-			if(count > DansGuardianReportLimit)
+			if (count > DansGuardianReportLimit)
 				continue;
 		}
 
@@ -159,19 +159,23 @@ void dansguardian_report(void)
 		output_html_link(fp_ou,url,100);
 		fprintf(fp_ou,"</td><td class=\"data2\">%s</td></tr>\n",rule);
 	}
-	fclose(fp_in);
+	if (fclose(fp_in)==EOF) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),dansguardian_in,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 
-	if(count>DansGuardianReportLimit && DansGuardianReportLimit>0)
+	if (count>DansGuardianReportLimit && DansGuardianReportLimit>0)
 		show_ignored_dansguardian(fp_ou,count-DansGuardianReportLimit);
 
 	fputs("</table></div>\n",fp_ou);
-	if (write_html_trailer(fp_ou)<0)
-		debuga(_("Write error in file \"%s\"\n"),report);
-	if (fclose(fp_ou)==EOF)
-		debuga(_("Failed to close file \"%s\": %s\n"),report,strerror(errno));
+	write_html_trailer(fp_ou);
+	if (fclose(fp_ou)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),report,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 
 	if (!KeepTempLog && unlink(dansguardian_in)) {
-		debuga(_("Cannot delete \"%s\": %s\n"),dansguardian_in,strerror(errno));
+		debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),dansguardian_in,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
