@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -61,7 +61,7 @@ to Dichotomic_Destroy().
 DichotomicObject Dichotomic_Create(void)
 {
 	DichotomicObject Obj;
-	
+
 	Obj=malloc(sizeof(*Obj));
 	if (!Obj)
 	{
@@ -83,7 +83,7 @@ void Dichotomic_Destroy(DichotomicObject *ObjPtr)
 {
 	DichotomicObject Obj;
 	int i;
-	
+
 	if (!ObjPtr || !*ObjPtr) return;
 	Obj=*ObjPtr;
 	*ObjPtr=NULL;
@@ -104,14 +104,14 @@ static int Dichotomic_FindKeyPos(DichotomicObject Obj,const char *key,bool *Foun
 	int down,up;
 	int middle=0;
 	int cmp=0;
-	
+
 	down=0;
 	up=Obj->NItems-1;
 	while (up>=down)
 	{
 		middle=(down+up)/2;
 		cmp=strcasecmp(key,Obj->Items[middle].Key);
-		if (!cmp) 
+		if (!cmp)
 		{
 			*Found=true;
 			return(middle);
@@ -141,7 +141,7 @@ bool Dichotomic_Insert(DichotomicObject Obj,const char *key, const char *value)
 	int Position;
 	bool Found;
 	int i;
-	
+
 	if (!Obj) return(false);
 	if (Obj->Items)
 	{
@@ -150,7 +150,7 @@ bool Dichotomic_Insert(DichotomicObject Obj,const char *key, const char *value)
 	}
 	else
 		Position=0;
-	
+
 	if (Obj->NItems>=Obj->NAllocated)
 	{
 		struct DichotomicItemStruct *Items;
@@ -158,12 +158,12 @@ bool Dichotomic_Insert(DichotomicObject Obj,const char *key, const char *value)
 		Items=realloc(Obj->Items,Obj->NAllocated*sizeof(*Items));
 		if (!Items)
 		{
-			debuga(_("Not enough memory to store the key/value pair %s/%s\n"),key,value);
+			debuga(__FILE__,__LINE__,_("Not enough memory to store the key/value pair %s/%s\n"),key,value);
 			exit(EXIT_FAILURE);
 		}
 		Obj->Items=Items;
 	}
-	
+
 	for (i=Obj->NItems ; i>Position ; i--)
 	{
 		Obj->Items[i].Key=Obj->Items[i-1].Key;
@@ -173,11 +173,11 @@ bool Dichotomic_Insert(DichotomicObject Obj,const char *key, const char *value)
 	Obj->Items[Position].Value=strdup(value);
 	if (!Obj->Items[Position].Key || !Obj->Items[Position].Value)
 	{
-		debuga(_("Not enough memory to store the key/value pair %s/%s\n"),key,value);
+		debuga(__FILE__,__LINE__,_("Not enough memory to store the key/value pair %s/%s\n"),key,value);
 		exit(EXIT_FAILURE);
 	}
 	Obj->NItems++;
-	
+
 	return(true);
 }
 
@@ -193,7 +193,7 @@ const char *Dichotomic_Search(DichotomicObject Obj,const char *key)
 {
 	int Position;
 	bool Found;
-	
+
 	if (!Obj) return(NULL);
 	if (Obj->NItems==0 || !Obj->Items) return(NULL);
 	Position=Dichotomic_FindKeyPos(Obj,key,&Found);
