@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -121,6 +121,8 @@ enum TextRefPos
 iconv_t localtoutf=(iconv_t)-1;
 #endif
 
+extern char GraphConfigFile[MAXLEN];
+
 static void Sarg_gdImageStringFT (struct GraphDataStruct *gdata, int fg, char *fontlist,
          double ptsize, double angle, int x, int y, const char *string,enum TextRefPos RefPos)
 {
@@ -141,7 +143,7 @@ static void Sarg_gdImageStringFT (struct GraphDataStruct *gdata, int fg, char *f
 		if (sslen>gdata->string_size) {
 			sstring = (char *)realloc(gdata->string,sslen);
 			if (!sstring) {
-				debuga(_("realloc error (%"PRIu64" bytes required)\n"),(uint64_t)sslen);
+				debuga(__FILE__,__LINE__,_("realloc error (%"PRIu64" bytes required)\n"),(uint64_t)sslen);
 				exit(EXIT_FAILURE);
 			}
 			gdata->string=(char *)sstring;
@@ -154,10 +156,7 @@ static void Sarg_gdImageStringFT (struct GraphDataStruct *gdata, int fg, char *f
 		str = string;
 		sstr = sstring;
 		if (iconv (localtoutf, (ICONV_CONST char **)&str, &slen, &sstr, &sslen)==-1) {
-			/* TRANSLATORS: First %s is the string that failed to convert. Second %s is the input
-			 * character encoding. Last %s is the reason for the failure.
-			 */
-			debugapos("grepday",_("iconv failed to convert string \"%s\" from %s to UTF-8: %s\n"),string,CharSet,strerror(errno));
+			debuga(__FILE__,__LINE__,_("iconv failed to convert string \"%s\" from %s to UTF-8: %s\n"),string,CharSet,strerror(errno));
 			sstring=(char *)string; //show something sensible on the graph
 		}
 	} else {
@@ -170,7 +169,7 @@ static void Sarg_gdImageStringFT (struct GraphDataStruct *gdata, int fg, char *f
 	if (RefPos!=TRP_BottomLeft) {
 		retval = gdImageStringFTEx (NULL, brect, fg, fontlist, ptsize, angle, 0, 0, sstring, gdFTEX_Unicode);
 		if (retval) {
-			debuga(_("libgd failed to calculate the bounding box of the text \"%s\": %s\n"),sstring,retval);
+			debuga(__FILE__,__LINE__,_("libgd failed to calculate the bounding box of the text \"%s\": %s\n"),sstring,retval);
 			exit(EXIT_FAILURE);
 		}
 		/*
@@ -237,7 +236,7 @@ static void Sarg_gdImageStringFT (struct GraphDataStruct *gdata, int fg, char *f
 	}
 	retval = gdImageStringFTEx (gdata->im, brect, fg, fontlist, ptsize, angle, x, y, sstring, gdFTEX_Unicode);
 	if (retval) {
-		debuga(_("libgd failed to render the text \"%s\": %s\n"),sstring,retval);
+		debuga(__FILE__,__LINE__,_("libgd failed to render the text \"%s\": %s\n"),sstring,retval);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -295,11 +294,11 @@ static int greport_compute_yaxis(struct PlotStruct *pdata,struct GraphDataStruct
 	double yscale;
 
 	if (pdata->ymin<0.) {
-		debuga(_("Minimum for Y scale of the graph is out of range: %"PRId64"\n"),(int64_t)pdata->ymin);
+		debuga(__FILE__,__LINE__,_("Minimum for Y scale of the graph is out of range: %"PRId64"\n"),(int64_t)pdata->ymin);
 		return(-1);
 	}
 	if (pdata->ymax<=0.) {
-		debuga(_("Maximum for Y scale of the graph is out of range: %"PRId64"\n"),(int64_t)pdata->ymax);
+		debuga(__FILE__,__LINE__,_("Maximum for Y scale of the graph is out of range: %"PRId64"\n"),(int64_t)pdata->ymax);
 		return(-1);
 	}
 
@@ -324,7 +323,7 @@ static int greport_compute_yaxis(struct PlotStruct *pdata,struct GraphDataStruct
 			break;
 
 		default:
-			debuga(_("Unknown type %d for Y axis scale\n"),pdata->ytype);
+			debuga(__FILE__,__LINE__,_("Unknown type %d for Y axis scale\n"),pdata->ytype);
 			return(-1);
 	}
 	gdata->YTickSpace=10;
@@ -378,7 +377,7 @@ static void greport_draw_yaxis(struct PlotStruct *pdata,struct GraphDataStruct *
 	xexterior=gdata->LeftGraph-10;
 	xinterior=gdata->LeftGraph;
 	xtick=gdata->LeftGraph-10-gdata->TickLength;
-	for(y=y0-gdata->YTickSpace ; y>=gdata->TopGraph ; y-=gdata->YTickSpace) {
+	for (y=y0-gdata->YTickSpace ; y>=gdata->TopGraph ; y-=gdata->YTickSpace) {
 		gdImageLine(gdata->im, xtick, y+yt, xexterior, y+yt, gdata->dimgray);
 		gdImageLine(gdata->im, xexterior, y+yt, xinterior, y, gdata->dimgray);
 		gdImageLine(gdata->im, xinterior, y, gdata->RightGraph, y, gdata->dimgray);
@@ -455,37 +454,37 @@ static void greport_plot(const struct userinfostruct *uinfo,struct PlotStruct *p
 	gdata.goldenrod = gdImageColorAllocate(gdata.im, 234, 234, 174);
 	gdata.goldenrod2 = gdImageColorAllocate(gdata.im, 207, 181, 59);
 
-	if(strcmp(GraphDaysBytesBarColor,"orange") == 0) {
+	if (strcmp(GraphDaysBytesBarColor,"orange") == 0) {
 		gdata.color1 = gdImageColorAllocate(gdata.im, 255, 233, 142);
 		gdata.color2 = gdImageColorAllocate(gdata.im, 220, 163, 72);
 		gdata.color3 = gdImageColorAllocate(gdata.im, 255, 198, 107);
 	}
-	else if(strcmp(GraphDaysBytesBarColor,"blue") == 0) {
+	else if (strcmp(GraphDaysBytesBarColor,"blue") == 0) {
 		gdata.color1 = gdImageColorAllocate(gdata.im, 62, 80, 167);
 		gdata.color2 = gdImageColorAllocate(gdata.im, 40, 51, 101);
 		gdata.color3 = gdImageColorAllocate(gdata.im, 57, 73, 150);
 	}
-	else if(strcmp(GraphDaysBytesBarColor,"green") == 0) {
+	else if (strcmp(GraphDaysBytesBarColor,"green") == 0) {
 		gdata.color1 = gdImageColorAllocate(gdata.im,120,166,129);
 		gdata.color2 = gdImageColorAllocate(gdata.im,84,113,82);
 		gdata.color3 = gdImageColorAllocate(gdata.im,158,223,167);
 	}
-	else if(strcmp(GraphDaysBytesBarColor,"yellow") == 0) {
+	else if (strcmp(GraphDaysBytesBarColor,"yellow") == 0) {
 		gdata.color1 = gdImageColorAllocate(gdata.im,185,185,10);
 		gdata.color2 = gdImageColorAllocate(gdata.im,111,111,10);
 		gdata.color3 = gdImageColorAllocate(gdata.im,166,166,10);
 	}
-	else if(strcmp(GraphDaysBytesBarColor,"brown") == 0) {
+	else if (strcmp(GraphDaysBytesBarColor,"brown") == 0) {
 		gdata.color1 = gdImageColorAllocate(gdata.im,97,45,27);
 		gdata.color2 = gdImageColorAllocate(gdata.im,60,30,20);
 		gdata.color3 = gdImageColorAllocate(gdata.im,88,41,26);
 	}
-	else if(strcmp(GraphDaysBytesBarColor,"red")  == 0){
+	else if (strcmp(GraphDaysBytesBarColor,"red")  == 0){
 		gdata.color1 = gdImageColorAllocate(gdata.im,185,10,10);
 		gdata.color2 = gdImageColorAllocate(gdata.im,111,10,10);
 		gdata.color3 = gdImageColorAllocate(gdata.im,166,10,10);
 	} else {
-		debuga(_("Unknown color \"%s\" requested for the graph. Using orange instead\n"),GraphDaysBytesBarColor);
+		debuga(__FILE__,__LINE__,_("Unknown color \"%s\" requested for the graph. Using orange instead\n"),GraphDaysBytesBarColor);
 		gdata.color1 = gdImageColorAllocate(gdata.im, 255, 233, 142);
 		gdata.color2 = gdImageColorAllocate(gdata.im, 220, 163, 72);
 		gdata.color3 = gdImageColorAllocate(gdata.im, 255, 198, 107);
@@ -537,7 +536,7 @@ static void greport_plot(const struct userinfostruct *uinfo,struct PlotStruct *p
 	greport_draw_yaxis(pdata,&gdata);
 
 	// X axis ticks and labels
-	for(y=1; y<=pdata->npoints; y++) {
+	for (y=1; y<=pdata->npoints; y++) {
 		x=gdata.LeftGraph-10+(int)((double)y*gdata.XScale+0.5);
 		gdImageLine(gdata.im, x, gdata.BottomDepth, x, gdata.BottomDepth+TickLength, gdata.dimgray);
 		sprintf(s,"%02d",y);
@@ -546,14 +545,14 @@ static void greport_plot(const struct userinfostruct *uinfo,struct PlotStruct *p
 
 	t = time(NULL);
 	local = localtime(&t);
-	if(DateFormat[0]=='u')
+	if (df=='u')
 		strftime(ftime, sizeof(ftime), "%b/%d/%Y %H:%M", local);
-	if(DateFormat[0]=='e')
+	if (df=='e')
 		strftime(ftime, sizeof(ftime), "%d/%b/%Y-%H:%M", local);
 
 	x=ImgXSize*5/12;
 	Sarg_gdImageStringFT(&gdata,gdata.darkblue,GraphFont,7,0.0,ImgXSize-10,ImgYSize-10,ftime,TRP_BottomRight);
-	if(ShowSargInfo) Sarg_gdImageStringFT(&gdata,gdata.darkblue,GraphFont,10,0.0,x,15,_("SARG, "),TRP_BottomRight);
+	if (ShowSargInfo) Sarg_gdImageStringFT(&gdata,gdata.darkblue,GraphFont,10,0.0,x,15,_("SARG, "),TRP_BottomRight);
 	Sarg_gdImageStringFT(&gdata,gdata.darkblue,GraphFont,10,0.0,x,15,Title,TRP_BottomLeft);
 	sprintf(warea,_("Period: %s"),period.text);
 	Sarg_gdImageStringFT(&gdata,gdata.darkblue,GraphFont,9,0.0,x,27,warea,TRP_BottomLeft);
@@ -614,16 +613,18 @@ static void greport_plot(const struct userinfostruct *uinfo,struct PlotStruct *p
 
 	if (snprintf(graph,sizeof(graph),"%s/%s/%s",outdirname,uinfo->filename,pdata->pngfile)>=sizeof(graph)) {
 		/* TRANSLATORS: The message is followed by the path that is too long. */
-		debuga(_("User name too long to manufacture file name "));
+		debuga(__FILE__,__LINE__,_("User name too long to manufacture file name "));
 		debuga_more("%s/%s/%s\n",outdirname,uinfo->filename,pdata->pngfile);
 		exit(EXIT_FAILURE);
 	}
-	if((pngout=fopen(graph,"wb"))==NULL) {
-		debugapos("grepday",_("Cannot open file \"%s\": %s\n"),graph,strerror(errno));
+	if ((pngout=fopen(graph,"wb"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),graph,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 	gdImagePng(gdata.im, pngout);
-	fclose(pngout);
+	if (fclose(pngout)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),graph,strerror(errno));
+	}
 	gdImageDestroy(gdata.im);
 
 	if (gdata.string) free(gdata.string);
@@ -635,33 +636,27 @@ void greport_prepare(void)
 {
 #ifdef HAVE_GD
 	if (!Graphs) {
-		if (debugz) {
-			/* TRANSLATORS: %s is the name of the configuration file providing
-			 * the graph option.
-			 */
-			debugaz(_("Graphs disabled as requested in \"%s\"\n"),ConfigFile);
-		}
+		if (debugz>=LogLevel_Process)
+			debugaz(__FILE__,__LINE__,_("Graphs disabled as requested in \"%s\"\n"),GraphConfigFile);
 		return;
 	}
 	if (GraphFont[0]=='\0') {
-		if (debugz) {
-			/* TRANSLATORS: %s is the name of the configuration file providing
-			 * the graph option.
-			 */
-			debugaz(_("Graphs disabled as no font names were provided in \"%s\"\n"),ConfigFile);
+		if (debugz>=LogLevel_Process) {
+			const char *File=(GraphConfigFile[0]) ? GraphConfigFile : ConfigFile;
+			debugaz(__FILE__,__LINE__,_("Graphs disabled as no font names were provided in \"%s\"\n"),File);
 		}
 		return;
 	}
 
-	if(access(GraphFont, R_OK) != 0) {
-		debugapos("grepday",_("Fontname \"%s\" not found\n"),GraphFont);
+	if (access(GraphFont, R_OK) != 0) {
+		debuga(__FILE__,__LINE__,_("Font name \"%s\" not found\n"),GraphFont);
 		exit(EXIT_FAILURE);
 	}
 
 #ifdef USE_ICONV
 	localtoutf = iconv_open ("UTF-8", CharSet);
 	if (localtoutf==(iconv_t)-1) {
-		debugapos("grepday",_("iconv cannot convert from %s to UTF-8: %s\n"),CharSet,strerror(errno));
+		debuga(__FILE__,__LINE__,_("iconv cannot convert from %s to UTF-8: %s\n"),CharSet,strerror(errno));
 	}
 #endif
 
@@ -683,22 +678,19 @@ void greport_day(const struct userinfostruct *uinfo)
 	struct getwordstruct gwarea;
 	struct PlotStruct pdata;
 
+	if (datetimeby==0) return;
+	if (!Graphs || GraphFont[0]=='\0') return;
 	if (snprintf(wdirname,sizeof(wdirname),"%s/%s.day",tmp,uinfo->filename)>=sizeof(wdirname)) {
-		debuga(_("User name too long to manufacture file name "));
+		debuga(__FILE__,__LINE__,_("User name too long to manufacture file name "));
 		debuga_more("%s/%s.day\n",tmp,uinfo->filename);
 		exit(EXIT_FAILURE);
 	}
-	if(access(wdirname, R_OK) != 0) {
-		return;
-	}
-	if(!Graphs || GraphFont[0]=='\0') {
-		if (!KeepTempLog && unlink(wdirname))
-			debuga(_("Cannot delete \"%s\": %s\n"),wdirname,strerror(errno));
+	if (access(wdirname, R_OK) != 0) {
 		return;
 	}
 
-	if((fp_in=fopen(wdirname,"r"))==NULL) {
-		debugapos("grepday",_("Cannot open file \"%s\": %s\n"),wdirname,strerror(errno));
+	if ((fp_in=fopen(wdirname,"r"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),wdirname,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -708,49 +700,48 @@ void greport_day(const struct userinfostruct *uinfo)
 		fixendofline(buf);
 		getword_start(&gwarea,buf);
 		if (getword_atoll(&llday,&gwarea,'/')<0) {
-			debuga(_("Invalid date in file \"%s\"\n"),wdirname);
+			debuga(__FILE__,__LINE__,_("Invalid date in file \"%s\"\n"),wdirname);
 			exit(EXIT_FAILURE);
 		}
 		day=(int)llday;
 		if (day<1 || day>31) continue;
 		if (getword_skip(20,&gwarea,'\t')<0 || getword_skip(20,&gwarea,'\t')<0) {
-			debuga(_("Invalid entry in file \"%s\"\n"),wdirname);
+			debuga(__FILE__,__LINE__,_("Invalid entry in file \"%s\"\n"),wdirname);
 			exit(EXIT_FAILURE);
 		}
 		if ((datetimeby & DATETIME_BYTE)!=0) {
 			if (getword_atoll(&bytes,&gwarea,'\t')<0) {
-				debuga(_("Invalid number of bytes in file \"%s\"\n"),wdirname);
+				debuga(__FILE__,__LINE__,_("Invalid number of bytes in file \"%s\"\n"),wdirname);
 				exit(EXIT_FAILURE);
 			}
 			bytespoints[day-1]+=bytes;
 		}
 		if ((datetimeby & DATETIME_ELAP)!=0) {
 			if (getword_atoll(&elap,&gwarea,'\0')<0) {
-				debuga(_("Invalid elapsed time in file \"%s\"\n"),wdirname);
+				debuga(__FILE__,__LINE__,_("Invalid elapsed time in file \"%s\"\n"),wdirname);
 				exit(EXIT_FAILURE);
 			}
 			elappoints[day-1]+=elap;
 		}
 	}
-	fclose(fp_in);
-	if (!KeepTempLog && unlink(wdirname)) {
-		debuga(_("Cannot delete \"%s\": %s\n"),wdirname,strerror(errno));
+	if (fclose(fp_in)==EOF) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),wdirname,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
 	if (snprintf(wdirname,sizeof(wdirname),"%s/%s/graph.html",outdirname,uinfo->filename)>=sizeof(wdirname)) {
-		debuga(_("User name too long to manufacture file name "));
+		debuga(__FILE__,__LINE__,_("User name too long to manufacture file name "));
 		debuga_more("%s/%s/%s\n",outdirname,uinfo->filename,"graph.html");
 		exit(EXIT_FAILURE);
 	}
 	if ((fp_ou=fopen(wdirname,"wt"))==NULL) {
-		debugapos("grepday",_("Cannot open file \"%s\": %s\n"),wdirname,strerror(errno));
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),wdirname,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 	write_html_head(fp_ou,(IndexTree == INDEX_TREE_DATE) ? 4 : 2,_("Graph report"),HTML_JS_NONE);
 
 	fputs("<table class=\"report\" cellpadding=\"0\" cellspacing=\"2\">\n", fp_ou);
-	if((datetimeby & DATETIME_BYTE)!=0) {
+	if ((datetimeby & DATETIME_BYTE)!=0) {
 		memset(&pdata,0,sizeof(pdata));
 		pdata.datapoints=bytespoints;
 		pdata.npoints=31;
@@ -763,7 +754,7 @@ void greport_day(const struct userinfostruct *uinfo)
 		greport_plot(uinfo,&pdata);
 		fprintf(fp_ou,"<tr><td><img src=\"%s\" alt=\"B\"></td></tr>\n",pdata.pngfile);
 	}
-	if((datetimeby & DATETIME_ELAP)!=0) {
+	if ((datetimeby & DATETIME_ELAP)!=0) {
 		memset(&pdata,0,sizeof(pdata));
 		pdata.datapoints=elappoints;
 		pdata.npoints=31;
@@ -778,10 +769,11 @@ void greport_day(const struct userinfostruct *uinfo)
 	}
 	fputs("</table>\n",fp_ou);
 
-	if (write_html_trailer(fp_ou)<0)
-		debuga(_("Write error in file \"%s\"\n"),wdirname);
-	if (fclose(fp_ou)==EOF)
-		debuga(_("Failed to close file \"%s\": %s\n"),wdirname,strerror(errno));
+	write_html_trailer(fp_ou);
+	if (fclose(fp_ou)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),wdirname,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 #endif //HAVE_GD
 
 	return;
