@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -27,11 +27,21 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
+//! Number of limits.
+int PerUserLimitsNumber=0;
+//! Log user's who downloaded more than the limit.
+struct PerUserLimitStruct PerUserLimits[MAX_USER_LIMITS];
+//! How to create a per user file.
+enum PerUserFileCreationEnum PerUserFileCreation=PUFC_Always;
+
 extern struct globalstatstruct globstat;
 
 void htmlrel(void)
 {
-	FILE *fp_in, *fp_ou, *fp_ip, *fp_ip2, *fp_usr;
+	FileObject *fp_in;
+	FileObject *fp_ip;
+	FILE *fp_ou;
+	FILE *fp_ip2;
 
 	long long int nnbytes=0, unbytes=0, tnbytes=0, totbytes=0, totbytes2=0;
 	long long int totelap=0, totelap2=0, nnelap=0, unelap=0, tnelap=0;
@@ -51,6 +61,8 @@ void htmlrel(void)
 	double perc=0, perc2=0, ouperc=0, inperc=0;
 	int count;
 	int cstatus;
+	int i;
+	unsigned int user_limit[(MAX_USER_LIMITS+sizeof(unsigned int)-1)/sizeof(unsigned int)];
 	bool have_denied_report;
 	const char *sort_field;
 	const char *sort_order;
@@ -61,13 +73,13 @@ void htmlrel(void)
 	userscan uscan;
 
 	if (snprintf(tmp2,sizeof(tmp2),"%s/sargtmp.int_unsort",tmp)>=sizeof(tmp2)) {
-		debuga(_("Path too long: "));
+		debuga(__FILE__,__LINE__,_("Path too long: "));
 		debuga_more("%s/sargtmp.int_unsort\n",tmp);
 		exit(EXIT_FAILURE);
 	}
 
 	if (snprintf(tmp3,sizeof(tmp3),"%s/sargtmp.int_log",tmp)>=sizeof(tmp3)) {
-		debuga(_("Path too long: "));
+		debuga(__FILE__,__LINE__,_("Path too long: "));
 		debuga_more("%s/sargtmp.int_log\n",tmp);
 		exit(EXIT_FAILURE);
 	}
@@ -79,14 +91,42 @@ void htmlrel(void)
 
 	sort_labels(&sort_field,&sort_order);
 
+	switch (PerUserFileCreation)
+	{
+		case PUFC_Always:
+			for (i=0 ; i<PerUserLimitsNumber ; i++) {
+				FILE *fp_usr=fopen(PerUserLimits[i].File,"wt");
+				if (fp_usr==NULL) {
+					debuga(__FILE__,__LINE__,_("Cannot create empty per_user_limit file \"%s\": %s\n"),PerUserLimits[i].File,
+						   strerror(errno));
+					exit(EXIT_FAILURE);
+				}
+				if (fclose(fp_usr)==EOF) {
+					debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),PerUserLimits[i].File,strerror(errno));
+					exit(EXIT_FAILURE);
+				}
+			}
+			break;
+
+		case PUFC_AsRequired:
+			for (i=0 ; i<PerUserLimitsNumber ; i++) {
+				if (access(PerUserLimits[i].File,R_OK)==0 && unlink(PerUserLimits[i].File)==-1) {
+					debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),PerUserLimits[i].File,
+						   strerror(errno));
+					exit(EXIT_FAILURE);
+				}
+			}
+			break;
+	}
+
 	uscan=userinfo_startscan();
 	if (uscan == NULL) {
-		debuga(_("Cannot enumerate the user list\n"));
+		debuga(__FILE__,__LINE__,_("Cannot enumerate the user list\n"));
 		exit(EXIT_FAILURE);
 	}
 	while ( (uinfo = userinfo_advancescan(uscan)) != NULL ) {
 		if (snprintf(warea,sizeof(warea),"%s/%s",outdirname,uinfo->filename)>=sizeof(warea)) {
-			debuga(_("Destination directory too long: "));
+			debuga(__FILE__,__LINE__,_("Path too long: "));
 			debuga_more("%s/%s\n",outdirname,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
@@ -97,44 +137,46 @@ void htmlrel(void)
 		}
 
 		if (access(warea, R_OK) != 0) {
-			if (mkdir(warea,0755)) {
-				debuga(_("Cannot create directory \"%s\": %s\n"),warea,strerror(errno));
+			if (PortableMkDir(warea,0755)) {
+				debuga(__FILE__,__LINE__,_("Cannot create directory \"%s\": %s\n"),warea,strerror(errno));
 				exit(EXIT_FAILURE);
 			}
 		}
 		tmpsort(uinfo);
 
 		if (snprintf(arqin,sizeof(arqin),"%s/htmlrel.txt",tmp)>=sizeof(arqin)) {
-			debuga(_("Path too long: "));
-			debuga_more("%s/htmlrel.txt\n",tmp);
+			debuga(__FILE__,__LINE__,_("Input file name too long: %s/htmlrel.txt\n"),tmp);
 			exit(EXIT_FAILURE);
 		}
-		if ((fp_in = fopen(arqin, "r")) == 0){
+		if ((fp_in = FileObject_Open(arqin)) == 0){
 			if (uinfo->no_report) continue;
-			debugapos("html3",_("Cannot open file \"%s\": %s\n"),arqin,strerror(errno));
+			debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),arqin,FileObject_GetLastOpenError());
 			exit(EXIT_FAILURE);
 		}
 
 		if (snprintf(arqou,sizeof(arqou),"%s/%s/%s.html",outdirname,uinfo->filename,uinfo->filename)>=sizeof(arqou)) {
-			debuga(_("Path too long: "));
+			debuga(__FILE__,__LINE__,_("Path too long: "));
 			debuga_more("%s/%s/%s.html\n",outdirname,uinfo->filename,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
 		if (snprintf(duser,sizeof(duser),"%s/denied_%s.html",outdirname,uinfo->filename)>=sizeof(duser)) {
-			debuga(_("Path too long: "));
+			debuga(__FILE__,__LINE__,_("Path too long: "));
 			debuga_more("%s/denied_%s.html\n",outdirname,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
-		if(access(duser, R_OK) != 0)
+		if (access(duser, R_OK) != 0)
 			have_denied_report=false;
 		else
 			have_denied_report=true;
 
 		if ((line=longline_create())==NULL) {
-			debuga(_("Not enough memory to read file \"%s\"\n"),arqin);
+			debuga(__FILE__,__LINE__,_("Not enough memory to read file \"%s\"\n"),arqin);
 			exit(EXIT_FAILURE);
 		}
 
+		for (i=0 ; i<sizeof(user_limit)/sizeof(user_limit[0]) ; i++)
+			user_limit[i]=0;
+
 		tnacc=0;
 		tnbytes=0;
 		tnelap=0;
@@ -143,44 +185,44 @@ void htmlrel(void)
 		while((buf=longline_read(fp_in,line))!=NULL) {
 			getword_start(&gwarea,buf);
 			if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
-				debuga(_("Invalid number of accesses in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid number of accesses in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			tnacc+=ltemp;
 			if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
-				debuga(_("Invalid download size in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid downloaded size in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			tnbytes+=ltemp;
 			if (getword_ptr(NULL,NULL,&gwarea,'\t')<0) {
-				debuga(_("Invalid url in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid url in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_skip(MAXLEN,&gwarea,'\t')<0) {
-				debuga(_("Invalid access code in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid access code in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
-				debuga(_("Invalid elapsed time in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid elapsed time in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			tnelap+=ltemp;
 			if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
-				debuga(_("Invalid in-cache volume in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid in-cache size in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			tnincache+=ltemp;
 			if (getword_atoll(&ltemp,&gwarea,'\n')<0) {
-				debuga(_("Invalid out-of-cache volume in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid out-of-cache size in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			tnoucache+=ltemp;
 		}
 
-		rewind(fp_in);
+		FileObject_Rewind(fp_in);
 
 		if ((fp_ou = fopen(arqou, "w")) == 0){
-			debugapos("html5",_("Cannot open file \"%s\": %s\n"),arqou,strerror(errno));
+			debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),arqou,strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 
@@ -207,25 +249,25 @@ void htmlrel(void)
 		if (SortTableJs[0]) fputs(" sorttable_alpha",fp_ou);
 		fprintf(fp_ou,"\">%s</th>",_("ACCESSED SITE"));
 
-		if((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
+		if ((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
 			fprintf(fp_ou,"<th class=\"header_l\">%s</th>",_("CONNECT"));
-		if((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
+		if ((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
 			fprintf(fp_ou,"<th class=\"header_l\">%s</th>",_("BYTES"));
-		if((UserReportFields & USERREPORTFIELDS_SETYB) != 0)
-			fprintf(fp_ou,"<th class=\"header_l\">%%%s</th>",_("BYTES"));
-		if((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0)
+		if ((UserReportFields & USERREPORTFIELDS_SETYB) != 0)
+			fprintf(fp_ou,"<th class=\"header_l\">%s</th>",_("%BYTES"));
+		if ((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0)
 			fprintf(fp_ou,"<th class=\"header_c\" colspan=\"2\">%s</th><th style=\"display:none;\"></th>",_("IN-CACHE-OUT"));
-		if((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
+		if ((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
 			fprintf(fp_ou,"<th class=\"header_l\">%s</th>",_("ELAPSED TIME"));
-		if((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
+		if ((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
 			fprintf(fp_ou,"<th class=\"header_l\">%s</th>",_("MILLISEC"));
-		if((UserReportFields & USERREPORTFIELDS_PTIME) != 0)
-			fprintf(fp_ou,"<th class=\"header_l\">%%%s</th>",_("TIME"));
+		if ((UserReportFields & USERREPORTFIELDS_PTIME) != 0)
+			fprintf(fp_ou,"<th class=\"header_l\">%s</th>",pgettext("duration","%TIME"));
 
 		fputs("</tr></thead>\n",fp_ou);
 
-		if(debug) {
-			debuga(_("Making report %s\n"),uinfo->id);
+		if (debug) {
+			debuga(__FILE__,__LINE__,_("Making report %s\n"),uinfo->id);
 		}
 		count=0;
 		arqip[0]='\0';
@@ -233,54 +275,54 @@ void htmlrel(void)
 		while((buf=longline_read(fp_in,line))!=NULL) {
 			getword_start(&gwarea,buf);
 			if (getword_atoll(&twork,&gwarea,'\t')<0) {
-				debuga(_("Invalid number of accesses in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid number of accesses in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_atoll(&nnbytes,&gwarea,'\t')<0) {
-				debuga(_("Invalid number of bytes in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid number of bytes in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_ptr(buf,&url,&gwarea,'\t')<0) {
-				debuga(_("Invalid url in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid url in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword(tmsg,sizeof(tmsg),&gwarea,'\t')<0) {
-				debuga(_("Invalid access code in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid access code in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_atoll(&nnelap,&gwarea,'\t')<0) {
-				debuga(_("Invalid elapsed time in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid elapsed time in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_atoll(&incache,&gwarea,'\t')<0) {
-				debuga(_("Invalid in-cache column in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid in-cache size in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 			if (getword_atoll(&oucache,&gwarea,'\n')<0) {
-				debuga(_("Invalid out-of-cache column in file \"%s\"\n"),arqin);
+				debuga(__FILE__,__LINE__,_("Invalid out-of-cache size in file \"%s\"\n"),arqin);
 				exit(EXIT_FAILURE);
 			}
 
-			if(UserReportLimit<=0 || count<=UserReportLimit) {
+			if (UserReportLimit<=0 || count<=UserReportLimit) {
 				fputs("<tr>",fp_ou);
 
-				if(IndexTree == INDEX_TREE_DATE)
+				if (IndexTree == INDEX_TREE_DATE)
 					sprintf(tmp6,"../%s",ImageFile);
 				else
 					strcpy(tmp6,"../../images");
 
-				if((ReportType & REPORT_TYPE_SITE_USER_TIME_DATE) != 0) {
-					url_to_file(url,siteind,sizeof(siteind));
-					fprintf(fp_ou,"<td class=\"data\"><a href=\"tt%s-%s.html\"><img src=\"%s/datetime.png\" title=\"%s\" alt=\"T\"></a></td>",uinfo->filename,siteind,tmp6,_("date/time report"));
+				if ((ReportType & REPORT_TYPE_SITE_USER_TIME_DATE) != 0) {
+					url_to_anchor(url,siteind,sizeof(siteind));
+					fprintf(fp_ou,"<td class=\"data\"><a href=\"tt.html#%s\"><img src=\"%s/datetime.png\" title=\"%s\" alt=\"T\"></a></td>",siteind,tmp6,_("date/time report"));
 				} else {
 					fprintf(fp_ou,"<td class=\"data\"></td>");
 				}
 
-				if(Privacy)
+				if (Privacy)
 					fprintf(fp_ou,"<td class=\"data2\"><span style=\"color:%s;\">%s</span></td>",PrivacyStringColor,PrivacyString);
 				else {
 					fputs("<td class=\"data2\">",fp_ou);
-					if(BlockIt[0]!='\0' && url[0]!=ALIAS_PREFIX) {
+					if (BlockIt[0]!='\0' && url[0]!=ALIAS_PREFIX) {
 						fprintf(fp_ou,"<a href=\"%s%s?url=",wwwDocumentRoot,BlockIt);
 						output_html_url(fp_ou,url);
 						fprintf(fp_ou,"\"><img src=\"%s/sarg-squidguard-block.png\"></a>&nbsp;",tmp6);
@@ -289,164 +331,168 @@ void htmlrel(void)
 					fputs("</td>",fp_ou);
 				}
 
-				if((UserReportFields & USERREPORTFIELDS_CONNECT) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_CONNECT) != 0) {
 					fputs("<td class=\"data\"",fp_ou);
 					if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%"PRId64"\"",(int64_t)twork);
 					fprintf(fp_ou,">%s</td>",fixnum(twork,1));
 				}
-				if((UserReportFields & USERREPORTFIELDS_BYTES) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_BYTES) != 0) {
 					fputs("<td class=\"data\"",fp_ou);
 					if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%"PRId64"\"",(int64_t)nnbytes);
 					fprintf(fp_ou,">%s</td>",fixnum(nnbytes,1));
 				}
-				if((UserReportFields & USERREPORTFIELDS_SETYB) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_SETYB) != 0) {
 					perc=(tnbytes) ? nnbytes * 100. / tnbytes : 0.;
 					fprintf(fp_ou,"<td class=\"data\">%3.2lf%%</td>",perc);
 				}
-				if((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0) {
 					inperc=(nnbytes) ? incache * 100. / nnbytes : 0.;
 					ouperc=(nnbytes) ? oucache * 100. / nnbytes : 0.;
 					fprintf(fp_ou,"<td class=\"data\">%3.2lf%%</td><td class=\"data\">%3.2lf%%</td>",inperc,ouperc);
 				}
-				if((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0) {
 					fputs("<td class=\"data\"",fp_ou);
 					if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%"PRId64"\"",(int64_t)nnelap);
 					fprintf(fp_ou,">%s</td>",buildtime(nnelap));
 				}
-				if((UserReportFields & USERREPORTFIELDS_MILISEC) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_MILISEC) != 0) {
 					fputs("<td class=\"data\"",fp_ou);
 					if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%"PRId64"\"",(int64_t)nnelap);
 					fprintf(fp_ou,">%s</td>",fixnum2(nnelap,1));
 				}
-				if((UserReportFields & USERREPORTFIELDS_PTIME) != 0) {
+				if ((UserReportFields & USERREPORTFIELDS_PTIME) != 0) {
 					perc2=(tnelap) ? nnelap * 100. / tnelap : 0.;
 					fprintf(fp_ou,"<td class=\"data\">%3.2lf%%</td>",perc2);
 				}
 
-				if(strncmp(tmsg,"OK",2) != 0)
+				if (strncmp(tmsg,"OK",2) != 0)
 					fprintf(fp_ou,"<td class=\"data\">%s</td>",_("DENIED"));
 
 				fputs("</tr>\n",fp_ou);
 				count++;
 			} else if ((ReportType & REPORT_TYPE_SITE_USER_TIME_DATE) != 0) {
-				url_to_file(url,siteind,sizeof(siteind));
-				snprintf(warea,sizeof(warea),"%s/%s/tt%s-%s.html",outdirname,uinfo->filename,uinfo->filename,siteind);
+				format_path(__FILE__, __LINE__, warea,sizeof(warea), "%s/%s/tt.html", outdirname, uinfo->filename);
 				if (unlink(warea)!=0) {
-					debuga(_("Cannot delete \"%s\": %s\n"),warea,strerror(errno));
+					debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),warea,strerror(errno));
 				}
 			}
 
-			if(iprel) {
+			if (iprel) {
 				if (snprintf(arqip,sizeof(arqip),"%s/%s.ip",tmp,uinfo->filename)>=sizeof(arqip)) {
-					debuga(_("Path too long: "));
+					debuga(__FILE__,__LINE__,_("Path too long: "));
 					debuga_more("%s/%s.ip\n",tmp,uinfo->filename);
 					exit(EXIT_FAILURE);
 				}
 
-				if ((fp_ip = fopen(arqip, "r")) == 0){
-					debugapos("html6",_("Cannot open file \"%s\": %s\n"),arqip,strerror(errno));
+				if ((fp_ip = FileObject_Open(arqip)) == 0){
+					debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),arqip,FileObject_GetLastOpenError());
 					exit(EXIT_FAILURE);
 				}
 
 				if ((fp_ip2 = MY_FOPEN(tmp2, "a")) == 0){
-					debugapos("html7",_("Cannot open file \"%s\": %s\n"),tmp2,strerror(errno));
+					debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),tmp2,strerror(errno));
 					exit(EXIT_FAILURE);
 				}
 
 				if ((line1=longline_create())==NULL) {
-					debuga(_("Not enough memory to read file \"%s\"\n"),arqip);
+					debuga(__FILE__,__LINE__,_("Not enough memory to read file \"%s\"\n"),arqip);
 					exit(EXIT_FAILURE);
 				}
 				while((buf=longline_read(fp_ip,line1))!=NULL) {
 					getword_start(&gwarea,buf);
 					if (getword(user_ip,sizeof(user_ip),&gwarea,'\t')<0) {
-						debuga(_("Invalid user IP in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid user IP in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (getword_ptr(buf,&user_url,&gwarea,'\t')<0) {
-						debuga(_("Invalid url in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid url in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (strncmp(user_url,url,strlen(url))!=0) continue;
 					if (getword_skip(15,&gwarea,'\t')<0) {
-						debuga(_("Invalid day in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid day in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (getword_skip(15,&gwarea,'\t')<0) {
-						debuga(_("Invalid time in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid time in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (getword_atoll(&userbytes,&gwarea,'\t')<0) {
-						debuga(_("Invalid size in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid size in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (getword_atoll(&userelap,&gwarea,'\0')<0) {
-						debuga(_("Invalid elapsed time in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid elapsed time in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					fprintf(fp_ip2,"%s\t%"PRIu64"\t%"PRIu64"\n",user_ip,(uint64_t)userbytes,(uint64_t)userelap);
 				}
 				longline_destroy(&line1);
 
-				fclose(fp_ip);
-				fclose(fp_ip2);
+				if (fclose(fp_ip2)==EOF) {
+					debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),tmp2,strerror(errno));
+					exit(EXIT_FAILURE);
+				}
+				if (FileObject_Close(fp_ip)) {
+					debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),arqip,FileObject_GetLastCloseError());
+					exit(EXIT_FAILURE);
+				}
 
 				if (snprintf(csort,sizeof(csort),"sort -n -t \"\t\" -T \"%s\" -k 1,1 -k 2,2 -o \"%s\" \"%s\"",tmp,tmp3,tmp2)>=sizeof(csort)) {
-					debuga(_("Command too long: "));
-					debuga_more("sort -n -t \"\t\" -T \"%s\" -k 1,1 -k 2,2 -o \"%s\" \"%s\"",tmp,tmp3,tmp2);
+					debuga(__FILE__,__LINE__,_("Sort command too long when sorting file \"%s\" to \"%s\"\n"),tmp2,tmp3);
 					exit(EXIT_FAILURE);
 				}
 				cstatus=system(csort);
 				if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
-					debuga(_("sort command return status %d\n"),WEXITSTATUS(cstatus));
-					debuga(_("sort command: %s\n"),csort);
+					debuga(__FILE__,__LINE__,_("sort command return status %d\n"),WEXITSTATUS(cstatus));
+					debuga(__FILE__,__LINE__,_("sort command: %s\n"),csort);
 					exit(EXIT_FAILURE);
 				}
 
-				if ((fp_ip = MY_FOPEN(tmp3, "r")) == 0) {
-					debugapos("html8",_("Cannot open file \"%s\": %s\n"),tmp3,strerror(errno));
+				if ((fp_ip = FileObject_Open(tmp3)) == 0) {
+					debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),tmp3,FileObject_GetLastOpenError());
 					exit(EXIT_FAILURE);
 				}
 
 				if (unlink(tmp2)) {
-					debuga(_("Cannot delete \"%s\": %s\n"),tmp2,strerror(errno));
+					debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),tmp2,strerror(errno));
 					exit(EXIT_FAILURE);
 				}
 
 				olduserip[0]='\0';
 
 				if ((line1=longline_create())==NULL) {
-					debuga(_("Not enough memory to read file \"%s\"\n"),arqip);
+					debuga(__FILE__,__LINE__,_("Not enough memory to read file \"%s\"\n"),arqip);
 					exit(EXIT_FAILURE);
 				}
 				while((buf=longline_read(fp_ip,line1))!=NULL) {
 					getword_start(&gwarea,buf);
 					if (getword(user_ip,sizeof(user_ip),&gwarea,'\t')<0) {
-						debuga(_("Invalid user IP in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid user IP in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (getword_atoll(&userbytes,&gwarea,'\t')<0) {
-						debuga(_("Invalid size in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid size in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
 					if (getword_atoll(&userelap,&gwarea,'\0')<0) {
-						debuga(_("Invalid elapsed time in file \"%s\"\n"),tmp3);
+						debuga(__FILE__,__LINE__,_("Invalid elapsed time in file \"%s\"\n"),tmp3);
 						exit(EXIT_FAILURE);
 					}
-					if(strcmp(user_ip,olduserip) != 0) {
+					if (strcmp(user_ip,olduserip) != 0) {
 						if (olduserip[0]!='\0') {
 							fprintf(fp_ou,"<tr><td></td><td class=\"data\">%s</td>",olduserip);
-							if((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
+							if ((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
 								fputs("<td></td>",fp_ou);
-							if((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
+							if ((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
 								fprintf(fp_ou,"<td class=\"data\">%s</td>",fixnum(unbytes,1));
-							if((UserReportFields & USERREPORTFIELDS_SETYB) != 0)
+							if ((UserReportFields & USERREPORTFIELDS_SETYB) != 0)
 								fputs("<td></td>",fp_ou);
-							if((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0)
+							if ((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0)
 								fputs("</td><td></td><td></td>",fp_ou);
-							if((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
+							if ((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
 								fprintf(fp_ou,"<td class=\"data\">%s</td>",buildtime(unelap));
-							if((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
+							if ((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
 								fprintf(fp_ou,"<td class=\"data\">%s</td>",fixnum2(unelap,1));
 							fputs("</tr>\n",fp_ou);
 						}
@@ -460,27 +506,30 @@ void htmlrel(void)
 					unelap+=userelap;
 				}
 
-				fclose(fp_ip);
+				if (FileObject_Close(fp_ip)) {
+					debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),tmp3,FileObject_GetLastCloseError());
+					exit(EXIT_FAILURE);
+				}
 				longline_destroy(&line1);
 
 				if (unlink(tmp3)) {
-					debuga(_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
+					debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
 					exit(EXIT_FAILURE);
 				}
 
 				if (olduserip[0]!='\0') {
 					fprintf(fp_ou,"<tr><td></td><td class=\"data\">%s</td>",olduserip);
-					if((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
+					if ((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
 						fputs("<td></td>",fp_ou);
-					if((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
+					if ((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
 						fprintf(fp_ou,"<td class=\"data\">%s</td>",fixnum(unbytes,1));
-					if((UserReportFields & USERREPORTFIELDS_SETYB) != 0)
+					if ((UserReportFields & USERREPORTFIELDS_SETYB) != 0)
 						fputs("<td></td>",fp_ou);
-					if((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0)
+					if ((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0)
 						fputs("</td><td></td><td></td>",fp_ou);
-					if((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
+					if ((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
 						fprintf(fp_ou,"<td class=\"data\">%s</td>",buildtime(unelap));
-					if((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
+					if ((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
 						fprintf(fp_ou,"<td class=\"data\">%s</td>",fixnum2(unelap,1));
 					fputs("</tr>\n",fp_ou);
 				}
@@ -490,77 +539,84 @@ void htmlrel(void)
 			unelap=0;
 		}
 
-		fclose(fp_in);
+		if (FileObject_Close(fp_in)) {
+			debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),arqin,FileObject_GetLastCloseError());
+			exit(EXIT_FAILURE);
+		}
 		longline_destroy(&line);
 
 		if (iprel && arqip[0]) {
 			if (!KeepTempLog && unlink(arqip)) {
-				debuga(_("Cannot delete \"%s\": %s\n"),arqip,strerror(errno));
+				debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),arqip,strerror(errno));
 				exit(EXIT_FAILURE);
 			}
 		}
 		if (!KeepTempLog && unlink(arqin)) {
-			debuga(_("Cannot delete \"%s\": %s\n"),arqin,strerror(errno));
+			debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),arqin,strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 
 		if ((UserReportFields & (USERREPORTFIELDS_TOTAL | USERREPORTFIELDS_AVERAGE)) != 0)
 			fputs("<tfoot>",fp_ou);
 
-		if((UserReportFields & USERREPORTFIELDS_TOTAL) != 0) {
+		if ((UserReportFields & USERREPORTFIELDS_TOTAL) != 0) {
 			fprintf(fp_ou,"<tr><th></th><th class=\"header_l\">%s</th>",_("TOTAL"));
-			if((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",fixnum(tnacc,1));
-			if((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",fixnum(tnbytes,1));
-			if((UserReportFields & USERREPORTFIELDS_SETYB) != 0) {
+			if ((UserReportFields & USERREPORTFIELDS_SETYB) != 0) {
 				perc=(totbytes) ? tnbytes *100. / totbytes :0.;
 				fprintf(fp_ou,"<th class=\"header_r\">%3.2lf%%</th>",perc);
 			}
-			if((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0) {
+			if ((UserReportFields & USERREPORTFIELDS_IN_CACHE_OUT) != 0) {
 				inperc=(tnbytes) ? tnincache * 100. / tnbytes : 0.;
 				ouperc=(tnbytes) ? tnoucache * 100. / tnbytes : 0.;
 				fprintf(fp_ou,"<th class=\"header_r\">%3.2lf%%</th><th class=\"header_r\">%3.2lf%%</th>",inperc,ouperc);
 			}
-			if((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",buildtime(tnelap));
-			if((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",fixnum2(tnelap,1));
-			if((UserReportFields & USERREPORTFIELDS_PTIME) != 0) {
+			if ((UserReportFields & USERREPORTFIELDS_PTIME) != 0) {
 				perc2=(totelap) ? tnelap * 100. / totelap : 0.;
 				fprintf(fp_ou,"<th class=\"header_r\">%3.2lf%%</th>",perc2);
 			}
 			fputs("</tr>\n",fp_ou);
 		}
 
-		if(PerUserLimit > 0) {
-			if ((int)(tnbytes/1000000LLU) > PerUserLimit) {
-				limit_flag=0;
-				if(access(PerUserLimitFile, R_OK) == 0) {
-					if((fp_usr = fopen(PerUserLimitFile, "r")) == 0) {
-						debugapos("html9",_("Cannot open file \"%s\": %s\n"),PerUserLimitFile,strerror(errno));
+		if (PerUserLimitsNumber>0) {
+			int limit=(int)(tnbytes/1000000LLU);
+			int maskid;
+			int mask;
+			for (i=0 ; i<PerUserLimitsNumber ; i++) {
+				maskid=i/sizeof(unsigned int);
+				mask=0x1U << (i % sizeof(unsigned int));
+				if (limit>PerUserLimits[i].Limit && (user_limit[maskid] & mask)==0) {
+					FILE *fp_usr;
+
+					if ((fp_usr = fopen(PerUserLimits[i].File, "at")) == 0) {
+						debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),PerUserLimits[i].File,strerror(errno));
 						exit(EXIT_FAILURE);
 					}
-					while(fgets(tmp6,sizeof(tmp6),fp_usr)!=NULL) {
-						fixendofline(tmp6);
-						if(strcmp(tmp6,uinfo->ip) == 0) {
-							limit_flag=1;
+					switch (PerUserLimits[i].Output)
+					{
+						case PUOE_UserId:
+							fprintf(fp_usr,"%s\n",uinfo->label);
+							break;
+						case PUOE_UserIp:
+							fprintf(fp_usr,"%s\n",uinfo->ip);
 							break;
-						}
 					}
-					fclose(fp_usr);
-				}
-
-				if(!limit_flag) {
-					if((fp_usr = fopen(PerUserLimitFile, "a")) == 0) {
-						debugapos("html10",_("Cannot open file \"%s\": %s\n"),PerUserLimitFile,strerror(errno));
+					if (fclose(fp_usr)==EOF) {
+						debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),PerUserLimits[i].File,strerror(errno));
 						exit(EXIT_FAILURE);
 					}
-					fprintf(fp_usr,"%s\n",uinfo->ip);
-					fclose(fp_usr);
+					user_limit[maskid]|=mask;
 
-					if(debug)
-						debuga(_("User %s limit exceeded (%d MB). Added to file \"%s\"\n"),uinfo->label,PerUserLimit,PerUserLimitFile);
+					if (debug)
+						debuga(__FILE__,__LINE__,_("Limit exceeded for user %s (%d MB). Added to file \"%s\"\n"),uinfo->label,
+							   PerUserLimits[i].Limit,PerUserLimits[i].File);
 				}
 			}
 		}
@@ -570,16 +626,16 @@ void htmlrel(void)
 			totelap2=totelap/ntotuser;
 
 			fprintf(fp_ou,"<tr><th></th><th class=\"header_l\">%s</th>",_("AVERAGE"));
-			if((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_CONNECT) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",fixnum(ttnacc/ntotuser,1));
-			if((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_BYTES) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",fixnum(totbytes2,1));
 			fprintf(fp_ou,"<th></th><th></th><th></th>");
-			if((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_USED_TIME) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",buildtime(totelap2));
-			if((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
+			if ((UserReportFields & USERREPORTFIELDS_MILISEC) != 0)
 				fprintf(fp_ou,"<th class=\"header_r\">%s</th>",fixnum2(totelap2,1));
-			if((UserReportFields & USERREPORTFIELDS_PTIME) != 0) {
+			if ((UserReportFields & USERREPORTFIELDS_PTIME) != 0) {
 				perc2 = (totelap) ? totelap2 * 100. / totelap : 0.;
 				fprintf(fp_ou,"<th class=\"header_r\">%3.2lf%%</th>",perc2);
 			}
@@ -590,10 +646,11 @@ void htmlrel(void)
 			fputs("</tfoot>",fp_ou);
 
 		fputs("</table></div>\n",fp_ou);
-		if (write_html_trailer(fp_ou)<0)
-			debuga(_("Write error in file \"%s\"\n"),arqou);
-		if (fclose(fp_ou)==EOF)
-			debuga(_("Failed to close file \"%s\": %s\n"),arqou,strerror(errno));
+		write_html_trailer(fp_ou);
+		if (fclose(fp_ou)==EOF) {
+			debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),arqou,strerror(errno));
+			exit(EXIT_FAILURE);
+		}
 
 		htaccess(uinfo);
 	}
