@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -90,7 +90,7 @@ static enum ip2name_retcode ip2name_dns(char *ip,int ip_len)
 	if (error==EAI_NONAME)
 		return(INRC_NotFound);
 	if (error!=0) {
-		debuga(_("IP to name resolution (getnameinfo) on IP address \"%s\" failed with error %d - %s\n"),ip,error,gai_strerror(error));
+		debuga(__FILE__,__LINE__,_("IP to name resolution (getnameinfo) on IP address \"%s\" failed with error %d - %s\n"),ip,error,gai_strerror(error));
 		return(INRC_Error);
 	}
 	safe_strcpy(ip,host,ip_len);
