@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -27,7 +27,7 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
-extern numlist hours;
+extern int hours[24];
 
 void report_day(const struct userinfostruct *uinfo)
 {
@@ -57,17 +57,18 @@ void report_day(const struct userinfostruct *uinfo)
 	struct getwordstruct gwarea;
 	struct tm t;
 
-	snprintf(wdirname,sizeof(wdirname),"%s/%s.day",tmp,uinfo->filename);
-	if(access(wdirname, R_OK) != 0) return;
+	if (datetimeby==0) return;
+	format_path(__FILE__, __LINE__, wdirname, sizeof(wdirname), "%s/%s.day", tmp, uinfo->filename);
+	if (access(wdirname, R_OK) != 0) return;
 
 	if (snprintf(arqout,sizeof(arqout),"%s/%s/d%s.html",outdirname,uinfo->filename,uinfo->filename)>=sizeof(arqout)) {
-		debuga(_("Path too long: "));
+		debuga(__FILE__,__LINE__,_("Path too long: "));
 		debuga_more("%s/%s/d%s.html\n",outdirname,uinfo->filename,uinfo->filename);
 		exit(EXIT_FAILURE);
 	}
 
-	if((fp_in=fopen(wdirname,"r"))==NULL) {
-		debugapos("repday",_("Cannot open file \"%s\": %s\n"),wdirname,strerror(errno));
+	if ((fp_in=fopen(wdirname,"r"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),wdirname,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -79,7 +80,7 @@ void report_day(const struct userinfostruct *uinfo)
 		fixendofline(buf);
 		getword_start(&gwarea,buf);
 		if (getword(data,sizeof(data),&gwarea,'\t')<0) {
-			debuga(_("Invalid date in file \"%s\"\n"),wdirname);
+			debuga(__FILE__,__LINE__,_("Invalid date in file \"%s\"\n"),wdirname);
 			exit(EXIT_FAILURE);
 		}
 		if (sscanf(data,"%d/%d/%d",&day,&month,&year)!=3) continue;
@@ -88,14 +89,14 @@ void report_day(const struct userinfostruct *uinfo)
 		for (dayidx=0 ; dayidx<ndaylist && daynum!=daylist[dayidx] ; dayidx++);
 		if (dayidx>=ndaylist) {
 			if (dayidx>=sizeof(daylist)/sizeof(*daylist)) {
-				debuga(_("Too many different dates in \"%s\"\n"),wdirname);
+				debuga(__FILE__,__LINE__,_("Too many different dates in file \"%s\"\n"),wdirname);
 				exit(EXIT_FAILURE);
 			}
 			daylist[ndaylist++]=daynum;
 		}
 
 		if (getword_atoll(&hour,&gwarea,'\t')<0) {
-			debuga(_("Invalid time in file \"%s\"\n"),wdirname);
+			debuga(__FILE__,__LINE__,_("Invalid time in file \"%s\"\n"),wdirname);
 			exit(EXIT_FAILURE);
 		}
 		ihour=(int)hour;
@@ -104,23 +105,26 @@ void report_day(const struct userinfostruct *uinfo)
 		if ((datetimeby & DATETIME_BYTE)!=0) {
 			colsep=((datetimeby & DATETIME_ELAP)!=0) ? '\t' : '\0';
 			if (getword_atoll(&bytes,&gwarea,colsep)<0) {
-				debuga(_("Invalid number of bytes in file \"%s\"\n"),wdirname);
+				debuga(__FILE__,__LINE__,_("Invalid number of bytes in file \"%s\"\n"),wdirname);
 				exit(EXIT_FAILURE);
 			}
 			tbytes[i]+=bytes;
 		}
 		if ((datetimeby & DATETIME_ELAP)!=0) {
 			if (getword_atoll(&elap,&gwarea,'\0')<0) {
-				debuga(_("Invalid elapsed time in file \"%s\"\n"),wdirname);
+				debuga(__FILE__,__LINE__,_("Invalid elapsed time in file \"%s\"\n"),wdirname);
 				exit(EXIT_FAILURE);
 			}
 			telap[i]+=elap;
 		}
 	}
-	fclose(fp_in);
+	if (fclose(fp_in)==EOF) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),wdirname,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 
-	if((fp_ou=fopen(arqout,"w"))==NULL) {
-		debugapos("repday",_("Cannot open file \"%s\": %s\n"),arqout,strerror(errno));
+	if ((fp_ou=fopen(arqout,"w"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),arqout,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -137,14 +141,18 @@ void report_day(const struct userinfostruct *uinfo)
 		dayidx++;
 	}
 
-	if((datetimeby & DATETIME_BYTE)!=0) {
+	if ((datetimeby & DATETIME_BYTE)!=0) {
 		label=_("BYTES");
 		fputs("<table class=\"report\" cellpadding=\"0\" cellspacing=\"2\">\n", fp_ou);
 		fputs("<tr><th class=\"header_c\"></th>",fp_ou);
-		for( i = 0; i < hours.len; i++ ) {
-			/* TRANSLATORS: It is an hour in the hourly report */
-			sprintf(data,_("%02dH"),hours.list[ i ]);
-			fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td>\n", data, label );
+		for (i=0 ; i<24 ; i++)
+		{
+			if (hours[i])
+			{
+				/* TRANSLATORS: It is an hour in the hourly report */
+				sprintf(data,_("%02dH"),i);
+				fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td>\n", data, label );
+			}
 		}
 		fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td></tr>\n", _("TOTAL"), label );
 
@@ -158,13 +166,17 @@ void report_day(const struct userinfostruct *uinfo)
 			strftime(data,sizeof(data),"%x",&t);
 			fprintf(fp_ou, "<tr><td class=\"data\">%s</td>\n", data );
 			tt=0;
-			for( i = 0; i < hours.len; i++ ) {
-				ihour=hours.list[i];
-				if (tbytes[daynum*24+ihour]>0) {
+			for (i=0 ; i<24 ; i++)
+			{
+				if (!hours[i]) continue;
+				ihour=i;
+				if (tbytes[daynum*24+ihour]>0)
+				{
 					fprintf(fp_ou, "<td class=\"data\">%s</td>\n",fixnum(tbytes[daynum*24+ihour],1));
 					tt+=tbytes[daynum*24+ihour];
 					tttime[ihour]+=tbytes[daynum*24+ihour];
-				} else
+				}
+				else
 					fputs("<td class=\"data\"></td>\n",fp_ou);
 			}
 			fprintf(fp_ou, "<td class=\"data\">%s</td></tr>\n",fixnum(tt,1));
@@ -172,31 +184,39 @@ void report_day(const struct userinfostruct *uinfo)
 
 		fprintf(fp_ou, "<tr><td class=\"header_l\">%s</td>\n", _("TOTAL") );
 		tt=0;
-		for( i = 0; i < hours.len; i++ ) {
-			if (tttime[i]>0) {
+		for (i=0 ; i<24 ; i++)
+		{
+			if (!hours[i]) continue;
+			if (tttime[i]>0)
+			{
 				fprintf(fp_ou, "<td class=\"header_r\">%s</td>\n",fixnum(tttime[i],1));
 				tt+=tttime[i];
-			} else
+			}
+			else
 				fputs("<td class=\"header_r\"></td>\n",fp_ou);
 		}
 		fprintf(fp_ou, "<td class=\"header_r\">%s</td></tr>\n",fixnum(tt,1));
 		fputs("</table>\n",fp_ou);
 #ifdef ENABLE_DOUBLE_CHECK_DATA
 		if (tt!=uinfo->nbytes) {
-			debuga(_("Total downloaded bytes is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
+			debuga(__FILE__,__LINE__,_("Total downloaded bytes is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
 						 (int64_t)tt,(int64_t)uinfo->nbytes,uinfo->label);
 			exit(EXIT_FAILURE);
 		}
 #endif
 	}
 
-	if((datetimeby & DATETIME_ELAP)!=0) {
+	if ((datetimeby & DATETIME_ELAP)!=0) {
 		label=_("H:M:S");
 		fputs("<table class=\"report\" cellpadding=\"0\" cellspacing=\"2\">\n", fp_ou);
 		fputs("<tr><th class=\"header_c\"></th>",fp_ou);
-		for( i = 0; i < hours.len; i++ ) {
-			sprintf(data,_("%02dH"),hours.list[ i ]);
-			fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td>\n", data, label );
+		for (i=0 ; i<24 ; i++)
+		{
+			if (hours[i])
+			{
+				sprintf(data,_("%02dH"),i);
+				fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td>\n", data, label );
+			}
 		}
 		fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td></tr>\n", _("TOTAL"), label );
 
@@ -210,8 +230,10 @@ void report_day(const struct userinfostruct *uinfo)
 			strftime(data,sizeof(data),"%x",&t);
 			fprintf(fp_ou, "<tr><td class=\"data\">%s</td>\n", data );
 			tt=0;
-			for( i = 0; i < hours.len; i++ ) {
-				ihour=hours.list[i];
+			for (i=0 ; i<24 ; i++)
+			{
+				if (!hours[i]) continue;
+				ihour=i;
 				if (telap[daynum*24+ihour]>0) {
 					fprintf(fp_ou, "<td class=\"data\">%s</td>\n",fixtime(telap[daynum*24+ihour]));
 					tt+=telap[daynum*24+ihour];
@@ -224,27 +246,32 @@ void report_day(const struct userinfostruct *uinfo)
 
 		fprintf(fp_ou, "<tr><td class=\"header_l\">%s</td>\n", _("TOTAL") );
 		tt=0;
-		for( i = 0; i < hours.len; i++ ) {
-			if (tttime[i]>0) {
+		for (i=0 ; i<24 ; i++)
+		{
+			if (!hours[i]) continue;
+			if (tttime[i]>0)
+			{
 				fprintf(fp_ou, "<td class=\"header_r\">%s</td>\n",fixtime(tttime[i]));
 				tt+=tttime[i];
-			} else
+			}
+			else
 				fputs("<td class=\"header_r\"></td>\n",fp_ou);
 		}
 		fprintf(fp_ou, "<td class=\"header_r\">%s</td></tr>\n",fixtime(tt));
 		fputs("</table>\n",fp_ou);
 #ifdef ENABLE_DOUBLE_CHECK_DATA
 		if (tt!=uinfo->elap) {
-			debuga(_("Total elapsed time is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
+			debuga(__FILE__,__LINE__,_("Total elapsed time is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
 						 (int64_t)tt,(int64_t)uinfo->elap,uinfo->label);
 			exit(EXIT_FAILURE);
 		}
 #endif
 	}
 
-	if (write_html_trailer(fp_ou)<0)
-		debuga(_("Write error in file \"%s\"\n"),arqout);
-	if (fclose(fp_ou)==EOF)
-		debuga(_("Failed to close file \"%s\": %s\n"),arqout,strerror(errno));
+	write_html_trailer(fp_ou);
+	if (fclose(fp_ou)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),arqout,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 	return;
 }
