@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -33,7 +33,8 @@ extern struct globalstatstruct globstat;
 
 void topsites(void)
 {
-	FILE *fp_in, *fp_ou;
+	FileObject *fp_in;
+	FILE *fp_ou;
 
 	char *buf;
 	char *url;
@@ -68,57 +69,58 @@ void topsites(void)
 	longline line;
 	struct generalitemstruct item;
 
-	if(Privacy) {
-		if (debugz) debugaz(_("Top sites report not produced because privacy option is on\n"));
+	if (Privacy) {
+		if (debugz>=LogLevel_Process) debugaz(__FILE__,__LINE__,_("Top sites report not produced because privacy option is on\n"));
 		return;
 	}
+	if (debugz>=LogLevel_Process)
+		debuga(__FILE__,__LINE__,_("Creating top sites report...\n"));
 
-	sprintf(general,"%s/sarg-general",outdirname);
-	sprintf(sites,"%s/sarg-sites",outdirname);
-	sprintf(general2,"%s/sarg-general2",outdirname);
-	sprintf(general3,"%s/sarg-general3",outdirname);
+	format_path(__FILE__, __LINE__, general, sizeof(general), "%s/sarg-general", outdirname);
+	format_path(__FILE__, __LINE__, sites, sizeof(sites) ,"%s/sarg-sites",outdirname);
+	format_path(__FILE__, __LINE__, general2, sizeof(general2), "%s/sarg-general2", outdirname);
+	format_path(__FILE__, __LINE__, general3, sizeof(general3), "%s/sarg-general3", outdirname);
 
-	sprintf(report,"%s/topsites.html",outdirname);
+	format_path(__FILE__, __LINE__, report, sizeof(report), "%s/topsites.html", outdirname);
 
 	if (snprintf(csort,sizeof(csort),"sort -t \"\t\" -k 4,4 -k 1,1 -o \"%s\" \"%s\"",general2,general)>=sizeof(csort)) {
-		debuga(_("Command too long: "));
-		debuga_more("sort -t \"\t\" -k 4,4 -k 1,1 -o \"%s\" \"%s\"",general2,general);
+		debuga(__FILE__,__LINE__,_("Sort command too long when sorting file \"%s\" to \"%s\"\n"),general,general2);
 		exit(EXIT_FAILURE);
 	}
 	cstatus=system(csort);
 	if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
-		debuga(_("sort command return status %d\n"),WEXITSTATUS(cstatus));
-		debuga(_("sort command: %s\n"),csort);
+		debuga(__FILE__,__LINE__,_("sort command return status %d\n"),WEXITSTATUS(cstatus));
+		debuga(__FILE__,__LINE__,_("sort command: %s\n"),csort);
 		exit(EXIT_FAILURE);
 	}
 
-	if((fp_in=fopen(general2,"r"))==NULL) {
-		debugapos("topsites",_("Cannot open file \"%s\": %s\n"),general2,strerror(errno));
-		debuga(_("sort command: %s\n"),csort);
+	if ((fp_in=FileObject_Open(general2))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),general2,FileObject_GetLastOpenError());
+		debuga(__FILE__,__LINE__,_("sort command: %s\n"),csort);
 		exit(EXIT_FAILURE);
 	}
 
-	if((fp_ou=fopen(general3,"w"))==NULL) {
-		debugapos("topsites",_("Cannot open file \"%s\": %s\n"),general3,strerror(errno));
+	if ((fp_ou=fopen(general3,"w"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),general3,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
 	if ((line=longline_create())==NULL) {
-		debuga(_("Not enough memory to read file \"%s\"\n"),general2);
+		debuga(__FILE__,__LINE__,_("Not enough memory to read file \"%s\"\n"),general2);
 		exit(EXIT_FAILURE);
 	}
 
 	while((buf=longline_read(fp_in,line))!=NULL) {
 		ger_read(buf,&item,general2);
-		if(item.total) continue;
+		if (item.total) continue;
 
-		if(!regs) {
+		if (!regs) {
 			url_len=strlen(item.url);
 			if (!ourl || url_len>=ourl_size) {
 				ourl_size=url_len+1;
 				ourl=realloc(ourl,ourl_size);
 				if (!ourl) {
-					debuga(_("Not enough memory to store the url\n"));
+					debuga(__FILE__,__LINE__,_("Not enough memory to store the url\n"));
 					exit(EXIT_FAILURE);
 				}
 			}
@@ -126,7 +128,7 @@ void topsites(void)
 			regs++;
 		}
 
-		if(strcmp(item.url,ourl) != 0) {
+		if (strcmp(item.url,ourl) != 0) {
 			/*
 			This complicated printf is due to Microsoft's inability to comply with any standard. Msvcrt is unable
 			to print a long long int unless it is exactly 64-bits long.
@@ -137,7 +139,7 @@ void topsites(void)
 				ourl_size=url_len+1;
 				ourl=realloc(ourl,ourl_size);
 				if (!ourl) {
-					debuga(_("Not enough memory to store the url\n"));
+					debuga(__FILE__,__LINE__,_("Not enough memory to store the url\n"));
 					exit(EXIT_FAILURE);
 				}
 			}
@@ -161,7 +163,10 @@ void topsites(void)
 		ttntime+=item.nelap;
 #endif
 	}
-	fclose(fp_in);
+	if (FileObject_Close(fp_in)) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),general2,FileObject_GetLastCloseError());
+		exit(EXIT_FAILURE);
+	}
 	longline_destroy(&line);
 
 	if (ourl) {
@@ -173,61 +178,63 @@ void topsites(void)
 		free(ourl);
 	}
 
-	fclose(fp_ou);
+	if (fclose(fp_ou)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),general3,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 
 #ifdef ENABLE_DOUBLE_CHECK_DATA
 	if (ttnacc!=globstat.nacc || ttnbytes!=globstat.nbytes || ttntime!=globstat.elap) {
-		debuga(_("Total statistics mismatch when reading %s to produce the top sites\n"),general2);
+		debuga(__FILE__,__LINE__,_("Total statistics mismatch when reading \"%s\" to produce the top sites\n"),general2);
 		exit(EXIT_FAILURE);
 	}
 #endif
 
 	if (!KeepTempLog && unlink(general2)) {
-		debuga(_("Cannot delete \"%s\": %s\n"),general2,strerror(errno));
+		debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),general2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	if((TopsitesSort & TOPSITE_SORT_CONNECT) != 0) {
+	if ((TopsitesSort & TOPSITE_SORT_CONNECT) != 0) {
 		sortf="-k 1,1 -k 2,2";
-	} else if((TopsitesSort & TOPSITE_SORT_BYTES) != 0) {
+	} else if ((TopsitesSort & TOPSITE_SORT_BYTES) != 0) {
 		sortf="-k 2,2 -k 1,1";
-	} else if((TopsitesSort & TOPSITE_SORT_TIME) != 0) {
+	} else if ((TopsitesSort & TOPSITE_SORT_TIME) != 0) {
 		sortf="-k 3,3";
-	} else if((TopsitesSort & TOPSITE_SORT_USER) != 0) {
+	} else if ((TopsitesSort & TOPSITE_SORT_USER) != 0) {
 		sortf="-k 4,4 -k 1,1 -k 2,2";
 	} else {
 		sortf="-k 2,2 -k 1,1"; //default is BYTES
 	}
-	if((TopsitesSort & TOPSITE_SORT_REVERSE) != 0) {
+	if ((TopsitesSort & TOPSITE_SORT_REVERSE) != 0) {
 		sortt="-r";
 	} else {
 		sortt="";
 	}
 
 	if (snprintf(csort,sizeof(csort),"sort -t \"\t\" %s -n %s -o \"%s\" \"%s\"",sortt,sortf,sites,general3)>=sizeof(csort)) {
-		debuga(_("Command too long: "));
-		debuga_more("sort -t \"\t\" %s -n %s -o \"%s\" \"%s\"",sortt,sortf,sites,general3);
+		debuga(__FILE__,__LINE__,_("Sort command too long when sorting file \"%s\" to \"%s\"\n"),general3,sites);
 		exit(EXIT_FAILURE);
 	}
 	cstatus=system(csort);
 	if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
-		debuga(_("sort command return status %d\n"),WEXITSTATUS(cstatus));
-		debuga(_("sort command: %s\n"),csort);
+		debuga(__FILE__,__LINE__,_("sort command return status %d\n"),WEXITSTATUS(cstatus));
+		debuga(__FILE__,__LINE__,_("sort command: %s\n"),csort);
 		exit(EXIT_FAILURE);
 	}
-	if((fp_in=fopen(sites,"r"))==NULL) {
-		debugapos("topsites",_("Cannot open file \"%s\": %s\n"),sites,strerror(errno));
-		debuga(_("sort command: %s\n"),csort);
+	if ((fp_in=FileObject_Open(sites))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),sites,FileObject_GetLastOpenError());
+		debuga(__FILE__,__LINE__,_("sort command: %s\n"),csort);
 		exit(EXIT_FAILURE);
 	}
 
 	if (!KeepTempLog && unlink(general3)) {
-		debuga(_("Cannot delete \"%s\": %s\n"),general3,strerror(errno));
+		debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),general3,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	if((fp_ou=fopen(report,"w"))==NULL) {
-		debugapos("topsites",_("Cannot open file \"%s\": %s\n"),report,strerror(errno));
+	if ((fp_ou=fopen(report,"w"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),report,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -255,7 +262,7 @@ void topsites(void)
 	/* TRANSLATORS: This is a column header showing the number of transfered bytes. */
 	_("BYTES"),
 	/* TRANSLATORS: This is a column header showing the time spent by the proxy processing the requests. */
-	_("TIME"),
+	pgettext("duration","TIME"),
 	/* TRANSLATORS: This is a column header showing the number of users who visited a sites. */
 	_("USERS"));
 
@@ -263,31 +270,27 @@ void topsites(void)
 	ntopsites = 0;
 
 	if ((line=longline_create())==NULL) {
-		debuga(_("Not enough memory to read file \"%s\"\n"),sites);
+		debuga(__FILE__,__LINE__,_("Not enough memory to read file \"%s\"\n"),sites);
 		exit(EXIT_FAILURE);
 	}
 
 	while(regs<TopSitesNum && (buf=longline_read(fp_in,line))!=NULL) {
 		getword_start(&gwarea,buf);
 		if (getword_atoll(&nacc,&gwarea,'\t')<0) {
-			debuga(_("Invalid number of accesses in file \"%s\"\n"),sites);
+			debuga(__FILE__,__LINE__,_("Invalid record in file \"%s\"\n"),sites);
 			exit(EXIT_FAILURE);
 		}
 		if (nacc == 0) continue;
-		if (getword_atoll(&nbytes,&gwarea,'\t')<0) {
-			debuga(_("Invalid number of bytes in file \"%s\"\n"),sites);
-			exit(EXIT_FAILURE);
-		}
-		if (getword_atoll(&ntime,&gwarea,'\t')<0) {
-			debuga(_("Invalid elapsed time in file \"%s\"\n"),sites);
+		if (getword_atoll(&nbytes,&gwarea,'\t')<0 || getword_atoll(&ntime,&gwarea,'\t')<0) {
+			debuga(__FILE__,__LINE__,_("Invalid record in file \"%s\"\n"),sites);
 			exit(EXIT_FAILURE);
 		}
 		if (getword_atoi(&nusers,&gwarea,'\t')<0) {
-			debuga(_("Invalid number of users in file \"%s\"\n"),sites);
+			debuga(__FILE__,__LINE__,_("Invalid number of users in file \"%s\"\n"),sites);
 			exit(EXIT_FAILURE);
 		}
 		if (getword_ptr(buf,&url,&gwarea,'\t')<0) {
-			debuga(_("Invalid url in file \"%s\"\n"),sites);
+			debuga(__FILE__,__LINE__,_("Invalid url in file \"%s\"\n"),sites);
 			exit(EXIT_FAILURE);
 		}
 
@@ -297,7 +300,7 @@ void topsites(void)
 
 		fprintf(fp_ou,"<tr><td class=\"data\">%d</td><td class=\"data2\">",++regs);
 
-		if(BlockIt[0] != '\0' && url[0]!=ALIAS_PREFIX) {
+		if (BlockIt[0] != '\0' && url[0]!=ALIAS_PREFIX) {
 			fprintf(fp_ou,"<a href=\"%s%s?url=\"",wwwDocumentRoot,BlockIt);
 			output_html_url(fp_ou,url);
 			fputs("\"><img src=\"../images/sarg-squidguard-block.png\"></a>&nbsp;",fp_ou);
@@ -317,14 +320,18 @@ void topsites(void)
 		if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%d\"",nusers);
 		fprintf(fp_ou,">%s</td></tr>\n",fixnum(nusers,1));
 	}
-	fclose(fp_in);
+	if (FileObject_Close(fp_in)) {
+		debuga(__FILE__,__LINE__,_("Read error in \"%s\": %s\n"),sites,FileObject_GetLastCloseError());
+		exit(EXIT_FAILURE);
+	}
 	longline_destroy(&line);
 
 	fputs("</table></div>\n",fp_ou);
-	if (write_html_trailer(fp_ou)<0)
-		debuga(_("Write error in file \"%s\"\n"),report);
-	if (fclose(fp_ou)==EOF)
-		debuga(_("Failed to close file \"%s\": %s\n"),report,strerror(errno));
+	write_html_trailer(fp_ou);
+	if (fclose(fp_ou)==EOF) {
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),report,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
 
 	return;
 }
