@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2013
+ *                                                            1998, 2015
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -46,14 +46,14 @@ The object must be freed with a call to day_cleanup().
 DayObject day_prepare(void)
 {
 	DayObject ddata;
-	
+
 	ddata=(DayObject)malloc(sizeof(*ddata));
 	if (!ddata)
 	{
-		debuga(_("Not enough memory to store the daily statistics\n"));
+		debuga(__FILE__,__LINE__,_("Not enough memory to store the daily statistics\n"));
 		exit(EXIT_FAILURE);
 	}
-	
+
 	return(ddata);
 }
 
@@ -100,17 +100,18 @@ void day_addpoint(DayObject ddata,const char *date, const char *time, long long
 	int dayidx;
 	int i;
 
+	if (!ddata) return;
 	if (sscanf(date,"%d/%d/%d",&day,&month,&year)!=3) {
-		debuga(_("Invalid date \"%s\" for the hourly statistics\n"),date);
+		debuga(__FILE__,__LINE__,_("Invalid date \"%s\" for the hourly statistics\n"),date);
 		exit(EXIT_FAILURE);
 	}
 	if (day<1 || day>31 || month<1 || month>12 || year>9999) {
-		debuga(_("Invalid date component in \"%s\" for the hourly statistics\n"),date);
+		debuga(__FILE__,__LINE__,_("Invalid date component in \"%s\" for the hourly statistics\n"),date);
 		exit(EXIT_FAILURE);
 	}
 	hour=atoi(time);
 	if (hour<0 || hour>=24) {
-		debuga(_("Invalid hour %d for the hourly statistics\n"),hour);
+		debuga(__FILE__,__LINE__,_("Invalid hour %d for the hourly statistics\n"),hour);
 		exit(EXIT_FAILURE);
 	}
 	daynum=(year*10000)+(month*100)+day;
@@ -118,7 +119,7 @@ void day_addpoint(DayObject ddata,const char *date, const char *time, long long
 	if (dayidx<0) {
 		dayidx=ddata->ndaylist++;
 		if (dayidx>=sizeof(ddata->daylist)/sizeof(*ddata->daylist)) {
-			debuga(_("Too many different dates for the hourly statistics\n"));
+			debuga(__FILE__,__LINE__,_("Too many different dates for the hourly statistics\n"));
 			exit(EXIT_FAILURE);
 		}
 		ddata->daylist[dayidx]=daynum;
@@ -145,21 +146,18 @@ void day_totalize(DayObject ddata,const char *tmp, const struct userinfostruct *
 	int daynum;
 	int dayidx;
 	char arqout[2048];
-#ifdef ENABLE_DOUBLE_CHECK_DATA
-	long long int tt=0;
-#endif
 
 	if (datetimeby==0) return;
 	if (!ddata) return;
 
 	if (snprintf(arqout,sizeof(arqout),"%s/%s.day",tmp,uinfo->filename)>=sizeof(arqout)) {
-		debuga(_("Path too long: "));
+		debuga(__FILE__,__LINE__,_("Path too long: "));
 		debuga_more("%s/%s%s\n",tmp,uinfo->filename,".day");
 		exit(EXIT_FAILURE);
 	}
 
-	if((fp_ou=fopen(arqout,"w"))==NULL) {
-		debugapos("totday",_("Cannot open file \"%s\": %s\n"),arqout,strerror(errno));
+	if ((fp_ou=fopen(arqout,"w"))==NULL) {
+		debuga(__FILE__,__LINE__,_("Cannot open file \"%s\": %s\n"),arqout,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -167,7 +165,7 @@ void day_totalize(DayObject ddata,const char *tmp, const struct userinfostruct *
 		if (ddata->bytes[i]==0 && ddata->elap[i]==0) continue;
 		dayidx=i/24;
 		if (dayidx>=sizeof(ddata->daylist)/sizeof(*ddata->daylist)) {
-			debuga(_("Invalid day index found in the hourly statistics\n"));
+			debuga(__FILE__,__LINE__,_("Invalid day index found in the hourly statistics\n"));
 			exit(EXIT_FAILURE);
 		}
 		hour=i%24;
@@ -179,21 +177,33 @@ void day_totalize(DayObject ddata,const char *tmp, const struct userinfostruct *
 		if ((datetimeby & DATETIME_BYTE)!=0) fprintf(fp_ou,"\t%"PRIu64"",(uint64_t)ddata->bytes[i]);
 		if ((datetimeby & DATETIME_ELAP)!=0) fprintf(fp_ou,"\t%"PRIu64"",(uint64_t)ddata->elap[i]);
 		fputs("\n",fp_ou);
-#ifdef ENABLE_DOUBLE_CHECK_DATA
-		tt+=ddata->bytes[i];
-#endif
 	}
 
 	if (fclose(fp_ou)==EOF) {
-		debuga(_("Failed to close file \"%s\": %s\n"),arqout,strerror(errno));
+		debuga(__FILE__,__LINE__,_("Write error in \"%s\": %s\n"),arqout,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
-#ifdef ENABLE_DOUBLE_CHECK_DATA
-	if (tt!=uinfo->nbytes) {
-		debuga(_("Total downloaded bytes is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
-					 (int64_t)tt,(int64_t)uinfo->nbytes,uinfo->label);
-		exit(EXIT_FAILURE);
-	}
-#endif
 	return;
 }
+
+/*!
+Delete the temporary file generated by day_totalize().
+
+\param uinfo The user whose daily statistics are to be deleted.
+*/
+void day_deletefile(const struct userinfostruct *uinfo)
+{
+	char arqout[2048];
+
+	if (KeepTempLog) return;
+
+	if (snprintf(arqout,sizeof(arqout),"%s/%s.day",tmp,uinfo->filename)>=sizeof(arqout)) {
+		debuga(__FILE__,__LINE__,_("Path too long: "));
+		debuga_more("%s/%s%s\n",tmp,uinfo->filename,".day");
+		exit(EXIT_FAILURE);
+	}
+
+	if (unlink(arqout))
+		debuga(__FILE__,__LINE__,_("Cannot delete \"%s\": %s\n"),arqout,strerror(errno));
+}
+
